"""Odds Portal venue model."""

# pylint: disable=duplicate-code,too-many-arguments
import datetime

import pytest_is_running
from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ...cache import MEMORY
from ..google.google_address_model import create_google_address_model
from ..venue_model import VERSION, VenueModel


def _create_oddsportal_venue_model(
    session: ScrapeSession,
    dt: datetime.datetime,
    venue: str,
    venue_town: str,
    venue_country: str,
    version: str,
) -> VenueModel | None:
    return VenueModel(
        identifier=venue,
        name=venue,
        address=create_google_address_model(
            query=", ".join([venue, venue_town, venue_country]), session=session, dt=dt
        ),
        is_grass=None,
        is_indoor=None,
        is_turf=None,
        is_dirt=None,
        is_hard=None,
        version=version,
    )


@MEMORY.cache(ignore=["session"])
def _cached_create_oddsportal_venue_model(
    session: ScrapeSession,
    dt: datetime.datetime,
    venue: str,
    venue_town: str,
    venue_country: str,
    version: str,
) -> VenueModel | None:
    return _create_oddsportal_venue_model(
        session=session,
        dt=dt,
        venue=venue,
        venue_town=venue_town,
        venue_country=venue_country,
        version=version,
    )


def create_oddsportal_venue_model(
    session: ScrapeSession,
    dt: datetime.datetime,
    venue: str,
    venue_town: str,
    venue_country: str,
) -> VenueModel | None:
    """Create odds portal venue model."""
    if not pytest_is_running.is_running() and dt < datetime.datetime.now().replace(
        tzinfo=dt.tzinfo
    ) - datetime.timedelta(days=7):
        return _cached_create_oddsportal_venue_model(
            session=session,
            dt=dt,
            venue=venue,
            venue_town=venue_town,
            venue_country=venue_country,
            version=VERSION,
        )
    with session.cache_disabled():
        return _create_oddsportal_venue_model(
            session=session,
            dt=dt,
            venue=venue,
            venue_town=venue_town,
            venue_country=venue_country,
            version=VERSION,
        )



"""Tests for the afltables coach model class."""
import os
import unittest
import datetime

import requests_mock
import requests_cache
from sportsball.data.afl.afltables.afl_afltables_coach_model import create_afl_afltables_coach_model


class TestAFLTablesCoachModel(unittest.TestCase):

    def setUp(self):
        self._session = requests_cache.CachedSession(backend="memory")
        self.dir = os.path.dirname(__file__)

    def test_name(self):
        with requests_mock.Mocker() as m:
            url = "https://afltables.com/afl/stats/coaches/goldcoast.html"
            with open(os.path.join(self.dir, "coaches_goldcoast.html"), "rb") as f:
                m.get(url, content=f.read())
            with open(os.path.join(self.dir, "Guy_McKenna.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/coaches/Guy_McKenna.html", content=f.read())
            coach_model = create_afl_afltables_coach_model(
                url=url,
                session=self._session,
                year=2012,
                dt=datetime.datetime(2012, 10, 10),
            )
            self.assertEqual(coach_model.name, "McKenna, Guy")

    def test_null_coach(self):
        with requests_mock.Mocker() as m:
            url = "https://afltables.com/afl/stats/coaches/fitzroy.html"
            with open(os.path.join(self.dir, "coaches_fitzroy.html"), "rb") as f:
                m.get(url, content=f.read())
            coach_model = create_afl_afltables_coach_model(
                url=url,
                session=self._session,
                year=1897,
                dt=datetime.datetime(2012, 10, 10),
            )
            self.assertIsNone(coach_model)

    def test_bad_year_range(self):
        with requests_mock.Mocker() as m:
            url = "https://afltables.com/afl/stats/coaches/collingwood.html"
            with open(os.path.join(self.dir, "coaches_collingwood.html"), "rb") as f:
                m.get(url, content=f.read())
            coach_model = create_afl_afltables_coach_model(
                url=url,
                session=self._session,
                year=1897,
                dt=datetime.datetime(2012, 10, 10),
            )
            self.assertIsNone(coach_model)

    def test_no_birth_date(self):
        with requests_mock.Mocker() as m:
            url = "https://afltables.com/afl/stats/coaches/geelong.html"
            with open(os.path.join(self.dir, "coaches_geelong.html"), "rb") as f:
                m.get(url, content=f.read())
            with open(os.path.join(self.dir, "Dave_Hickinbotham.html"), "rb") as f:
                m.get("https://afltables.com/afl/stats/coaches/Dave_Hickinbotham.html", content=f.read())
            coach_model = create_afl_afltables_coach_model(
                url=url,
                session=self._session,
                year=1910,
                dt=datetime.datetime(2012, 10, 10),
            )
            self.assertIsNone(coach_model.birth_date)



"""AFL AFL odds model."""

import datetime

from ...bet import Bet
from ...team_model import OddsModel
from .afl_afl_bookie_model import create_afl_afl_bookie_model


def create_afl_afl_odds_model(odds: float) -> OddsModel:
    """Create an odds model based off aus sports betting."""
    bookie = create_afl_afl_bookie_model()
    return OddsModel(
        odds=odds,
        bookie=bookie,
        dt=datetime.datetime.now(),
        canonical=True,
        bet=str(Bet.WIN),
    )  # pyright: ignore



"""NBA NBA API game model."""

# pylint: disable=too-many-arguments,line-too-long,duplicate-code
import datetime

import pandas as pd
import pytest_is_running
import pytz
import requests_cache
from dateutil.parser import parse

from ....cache import MEMORY
from ...game_model import VERSION, GameModel
from ...league import League
from ...season_type import SeasonType
from .nba_nba_team_model import create_nba_nba_team_model

_SEASON_TYPE_MAP = {
    "1": SeasonType.PRESEASON,
    "2": SeasonType.REGULAR,
    "3": SeasonType.OFFSEASON,
    "4": SeasonType.POSTSEASON,
    "5": SeasonType.POSTSEASON,
    "6": SeasonType.OFFSEASON,
}


def _create_nba_nba_game_model(
    row: pd.Series,
    league: League,
    week: int,
    game_number: int,
    session: requests_cache.CachedSession,
    league_id: str,
    version: str,
) -> GameModel:
    season_id = row["SEASON_ID"]
    dt = pytz.timezone("EST").localize(parse(row["GAME_DATE"]))
    return GameModel(
        dt=dt,
        week=week,
        game_number=game_number,
        venue=None,
        teams=[
            x
            for x in [
                create_nba_nba_team_model(row, True, session, dt, league, league_id),  # type: ignore
                create_nba_nba_team_model(row, False, session, dt, league, league_id),  # type: ignore
            ]
            if x is not None
        ],
        end_dt=None,
        attendance=None,
        league=str(league),
        year=int(season_id[1:]),
        season_type=_SEASON_TYPE_MAP[season_id[0]],
        postponed=None,
        play_off=None,
        distance=None,
        dividends=[],
        pot=None,
        umpires=[],
        version=version,
        best_of=None,
    )


@MEMORY.cache(ignore=["session"])
def _cached_create_nba_nba_game_model(
    row: pd.Series,
    league: League,
    week: int,
    game_number: int,
    session: requests_cache.CachedSession,
    league_id: str,
    version: str,
) -> GameModel:
    """Create a game model from NBA API."""
    return _create_nba_nba_game_model(
        row,
        league,
        week,
        game_number,
        session,
        league_id,
        version=version,
    )


def create_nba_nba_game_model(
    row: pd.Series,
    league: League,
    week: int,
    game_number: int,
    session: requests_cache.CachedSession,
    dt: datetime.datetime,
    league_id: str,
) -> GameModel:
    """Create a game model from NBA API."""
    if (
        not pytest_is_running.is_running()
        and dt < datetime.datetime.now() - datetime.timedelta(days=7)
    ):
        return _cached_create_nba_nba_game_model(
            row,
            league,
            week,
            game_number,
            session,
            league_id,
            version=VERSION,
        )
    with session.cache_disabled():
        return _create_nba_nba_game_model(
            row,
            league,
            week,
            game_number,
            session,
            league_id,
            version=VERSION,
        )



"""Venue model from wikipedia information."""

# pylint: disable=duplicate-code
import logging

import requests
import wikipediaapi  # type: ignore

from ... import __VERSION__
from ...cache import MEMORY
from ..google.google_address_model import get_venue_db
from ..venue_model import VenueModel

WIKI_WIKI = wikipediaapi.Wikipedia(user_agent=f"sportsball ({__VERSION__})")


@MEMORY.cache(ignore=["session"])
def create_wikipedia_venue_model(
    session: requests.Session, latitude: float, longitude: float, version: str
) -> VenueModel | None:
    """Create a venue model by looking up the venue on wikipedia."""
    global WIKI_WIKI
    WIKI_WIKI._session = session

    venue_db = get_venue_db()
    for venue in venue_db["venues"].values():
        if venue["lat"] == latitude and venue["lng"] == longitude:
            if "wiki" not in venue:
                logging.warning("wiki not found for %f, %f", latitude, longitude)
            else:
                wikipage = venue["wiki"]
                if wikipage is not None:
                    page_py = WIKI_WIKI.page(wikipage)
                    return VenueModel(
                        identifier=wikipage,
                        name=page_py.title,
                        address=None,
                        is_grass=None,
                        is_indoor=None,
                        is_turf=None,
                        is_dirt=None,
                        is_hard=None,
                        version=version,
                    )
            break
    return None



"""Openmeteo weather model."""

# pylint: disable=too-many-statements,too-many-locals,line-too-long,duplicate-code,too-many-arguments,too-many-positional-arguments
import datetime
import struct

import openmeteo_requests  # type: ignore
import pandas as pd
import pytest_is_running
import pytz
import requests
import requests_cache
from openmeteo_requests.Client import OpenMeteoRequestsError  # type: ignore
from openmeteo_requests.Client import WeatherApiResponse

from ....cache import MEMORY
from ...weather_model import VERSION, WeatherModel


def _parse_openmeteo(
    responses: list[WeatherApiResponse], tz: str, dt: datetime.datetime, version: str
) -> WeatherModel | None:
    # pylint: disable=broad-exception-caught
    if not responses:
        return None
    response = responses[0]
    try:
        hourly = response.Hourly()
        daily = response.Daily()
    except Exception:
        return None
    if hourly is None:
        raise ValueError("hourly is null.")
    if daily is None:
        raise ValueError("daily is null")
    dt_index = pd.date_range(
        start=pd.to_datetime(hourly.Time(), unit="s"),
        end=pd.to_datetime(hourly.TimeEnd(), unit="s"),
        freq=pd.Timedelta(seconds=hourly.Interval()),
        inclusive="left",
        tz=tz,
    )
    try:
        row_count = min(
            len(hourly.Variables(0).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(1).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(2).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(3).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(4).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(5).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(6).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(7).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(8).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(9).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(10).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(11).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(12).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(13).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(14).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(15).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(16).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(17).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(18).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(19).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(20).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(21).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(22).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(23).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(24).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(25).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(26).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(27).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(28).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(29).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(30).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(31).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(32).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(33).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(34).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(35).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(36).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(37).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(38).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(39).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(40).ValuesAsNumpy()),  # type: ignore
            len(hourly.Variables(41).ValuesAsNumpy()),  # type: ignore
            len(dt_index),
        )
        hourly_df = pd.DataFrame(
            index=dt_index[:row_count],  # type: ignore
            data={
                "temperature_2m": hourly.Variables(0).ValuesAsNumpy()[:row_count],  # type: ignore
                "relative_humidity_2m": hourly.Variables(1).ValuesAsNumpy()[:row_count],  # type: ignore
                "dew_point_2m": hourly.Variables(2).ValuesAsNumpy()[:row_count],  # type: ignore
                "apparent_temperature": hourly.Variables(3).ValuesAsNumpy()[:row_count],  # type: ignore
                "precipitation_probability": hourly.Variables(4).ValuesAsNumpy()[  # type: ignore
                    :row_count
                ],  # type: ignore
                "precipitation": hourly.Variables(5).ValuesAsNumpy()[:row_count],  # type: ignore
                "rain": hourly.Variables(6).ValuesAsNumpy()[:row_count],  # type: ignore
                "showers": hourly.Variables(7).ValuesAsNumpy()[:row_count],  # type: ignore
                "snowfall": hourly.Variables(8).ValuesAsNumpy()[:row_count],  # type: ignore
                "snow_depth": hourly.Variables(9).ValuesAsNumpy()[:row_count],  # type: ignore
                "weather_code": hourly.Variables(10).ValuesAsNumpy()[:row_count],  # type: ignore
                "pressure_msl": hourly.Variables(11).ValuesAsNumpy()[:row_count],  # type: ignore
                "surface_pressure": hourly.Variables(12).ValuesAsNumpy()[:row_count],  # type: ignore
                "cloud_cover": hourly.Variables(13).ValuesAsNumpy()[:row_count],  # type: ignore
                "cloud_cover_low": hourly.Variables(14).ValuesAsNumpy()[:row_count],  # type: ignore
                "cloud_cover_mid": hourly.Variables(15).ValuesAsNumpy()[:row_count],  # type: ignore
                "cloud_cover_high": hourly.Variables(16).ValuesAsNumpy()[:row_count],  # type: ignore
                "visibility": hourly.Variables(17).ValuesAsNumpy()[:row_count],  # type: ignore
                "evapotranspiration": hourly.Variables(18).ValuesAsNumpy()[:row_count],  # type: ignore
                "reference_evapotranspiration": hourly.Variables(19).ValuesAsNumpy()[  # type: ignore
                    :row_count
                ],  # type: ignore
                "vapour_pressure_deficit": hourly.Variables(20).ValuesAsNumpy()[  # type: ignore
                    :row_count
                ],  # type: ignore
                "wind_speed_10m": hourly.Variables(21).ValuesAsNumpy()[:row_count],  # type: ignore
                "wind_speed_80m": hourly.Variables(22).ValuesAsNumpy()[:row_count],  # type: ignore
                "wind_speed_120m": hourly.Variables(23).ValuesAsNumpy()[:row_count],  # type: ignore
                "wind_speed_180m": hourly.Variables(24).ValuesAsNumpy()[:row_count],  # type: ignore
                "wind_direction_10m": hourly.Variables(25).ValuesAsNumpy()[:row_count],  # type: ignore
                "wind_direction_80m": hourly.Variables(26).ValuesAsNumpy()[:row_count],  # type: ignore
                "wind_direction_120m": hourly.Variables(27).ValuesAsNumpy()[:row_count],  # type: ignore
                "wind_direction_180m": hourly.Variables(28).ValuesAsNumpy()[:row_count],  # type: ignore
                "wind_gusts": hourly.Variables(29).ValuesAsNumpy()[:row_count],  # type: ignore
                "temperature_80m": hourly.Variables(30).ValuesAsNumpy()[:row_count],  # type: ignore
                "temperature_120m": hourly.Variables(31).ValuesAsNumpy()[:row_count],  # type: ignore
                "temperature_180m": hourly.Variables(32).ValuesAsNumpy()[:row_count],  # type: ignore
                "soil_temperature": hourly.Variables(33).ValuesAsNumpy()[:row_count],  # type: ignore
                "soil_temperature_6cm": hourly.Variables(34).ValuesAsNumpy()[  # type: ignore
                    :row_count
                ],  # type: ignore
                "soil_temperature_18cm": hourly.Variables(35).ValuesAsNumpy()[  # type: ignore
                    :row_count
                ],  # type: ignore
                "soil_temperature_54cm": hourly.Variables(36).ValuesAsNumpy()[  # type: ignore
                    :row_count
                ],  # type: ignore
                "soil_moisture": hourly.Variables(37).ValuesAsNumpy()[:row_count],  # type: ignore
                "soil_moisture_1cm": hourly.Variables(38).ValuesAsNumpy()[:row_count],  # type: ignore
                "soil_moisture_3cm": hourly.Variables(39).ValuesAsNumpy()[:row_count],  # type: ignore
                "soil_moisture_9cm": hourly.Variables(40).ValuesAsNumpy()[:row_count],  # type: ignore
                "soil_moisture_27cm": hourly.Variables(41).ValuesAsNumpy()[:row_count],  # type: ignore
            },
        )

        dt_index = pd.date_range(
            start=pd.to_datetime(daily.Time(), unit="s"),
            end=pd.to_datetime(daily.TimeEnd(), unit="s"),
            freq=pd.Timedelta(seconds=daily.Interval()),
            inclusive="left",
            tz=tz,
        )
        row_count = min(
            len(daily.Variables(0).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(1).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(2).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(3).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(4).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(5).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(6).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(7).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(8).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(9).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(10).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(11).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(12).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(13).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(14).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(15).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(16).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(17).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(18).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(19).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(20).ValuesAsNumpy()),  # type: ignore
            len(daily.Variables(21).ValuesAsNumpy()),  # type: ignore
            len(dt_index),
        )
        daily_df = pd.DataFrame(
            index=dt_index[:row_count],  # type: ignore
            data={
                "weather_code": daily.Variables(0).ValuesAsNumpy()[:row_count],  # type: ignore
                "temperature_2m_max": daily.Variables(1).ValuesAsNumpy()[:row_count],  # type: ignore
                "temperature_2m_min": daily.Variables(2).ValuesAsNumpy()[:row_count],  # type: ignore
                "apparent_temperature_max": daily.Variables(3).ValuesAsNumpy()[  # type: ignore
                    :row_count
                ],  # type: ignore
                "apparent_temperature_min": daily.Variables(4).ValuesAsNumpy()[  # type: ignore
                    :row_count
                ],  # type: ignore
                "sunrise": daily.Variables(5).ValuesAsNumpy()[:row_count],  # type: ignore
                "sunset": daily.Variables(6).ValuesAsNumpy()[:row_count],  # type: ignore
                "daylight_duration": daily.Variables(7).ValuesAsNumpy()[:row_count],  # type: ignore
                "sunshine_duration": daily.Variables(8).ValuesAsNumpy()[:row_count],  # type: ignore
                "uv_index_max": daily.Variables(9).ValuesAsNumpy()[:row_count],  # type: ignore
                "uv_index_clear_sky_max": daily.Variables(10).ValuesAsNumpy()[  # type: ignore
                    :row_count
                ],  # type: ignore
                "rain_sum": daily.Variables(11).ValuesAsNumpy()[:row_count],  # type: ignore
                "showers_sum": daily.Variables(12).ValuesAsNumpy()[:row_count],  # type: ignore
                "snowfall_sum": daily.Variables(13).ValuesAsNumpy()[:row_count],  # type: ignore
                "precipitation_sum": daily.Variables(14).ValuesAsNumpy()[:row_count],  # type: ignore
                "precipitation_hours": daily.Variables(15).ValuesAsNumpy()[:row_count],  # type: ignore
                "precipitation_probability_max": daily.Variables(16).ValuesAsNumpy()[  # type: ignore
                    :row_count
                ],  # type: ignore
                "wind_speed_10m_max": daily.Variables(17).ValuesAsNumpy()[:row_count],  # type: ignore
                "wind_gusts_10m_max": daily.Variables(18).ValuesAsNumpy()[:row_count],  # type: ignore
                "wind_direction_10m_dominant": daily.Variables(19).ValuesAsNumpy()[  # type: ignore
                    :row_count
                ],  # type: ignore
                "shortwave_radiation_sum": daily.Variables(20).ValuesAsNumpy()[  # type: ignore
                    :row_count
                ],  # type: ignore
                "et0_fao_evapotranspiration": daily.Variables(21).ValuesAsNumpy()[  # type: ignore
                    :row_count
                ],  # type: ignore
            },
        )

        dt = dt.replace(tzinfo=None)
        timezone = pytz.timezone(tz)
        dt = timezone.localize(dt)

        hourly_idx = hourly_df.index.get_indexer([dt], method="nearest")[0]
        temperature = hourly_df.iloc[hourly_idx]["temperature_2m"]  # type: ignore
        relative_humidity = hourly_df.iloc[hourly_idx]["relative_humidity_2m"]  # type: ignore
        dew_point_2m = hourly_df.iloc[hourly_idx]["dew_point_2m"]  # type: ignore
        apparent_temperature = hourly_df.iloc[hourly_idx]["apparent_temperature"]  # type: ignore
        precipitation_probability = hourly_df.iloc[hourly_idx][
            "precipitation_probability"
        ]  # type: ignore
        precipitation = hourly_df.iloc[hourly_idx]["precipitation"]  # type: ignore
        rain = hourly_df.iloc[hourly_idx]["rain"]  # type: ignore
        showers = hourly_df.iloc[hourly_idx]["showers"]  # type: ignore
        snowfall = hourly_df.iloc[hourly_idx]["snowfall"]  # type: ignore
        snow_depth = hourly_df.iloc[hourly_idx]["snow_depth"]  # type: ignore
        weather_code = hourly_df.iloc[hourly_idx]["weather_code"]  # type: ignore
        pressure_msl = hourly_df.iloc[hourly_idx]["pressure_msl"]  # type: ignore
        surface_pressure = hourly_df.iloc[hourly_idx]["surface_pressure"]  # type: ignore
        cloud_cover = hourly_df.iloc[hourly_idx]["cloud_cover"]  # type: ignore
        cloud_cover_low = hourly_df.iloc[hourly_idx]["cloud_cover_low"]  # type: ignore
        cloud_cover_mid = hourly_df.iloc[hourly_idx]["cloud_cover_mid"]  # type: ignore
        cloud_cover_high = hourly_df.iloc[hourly_idx]["cloud_cover_high"]  # type: ignore
        visibility = hourly_df.iloc[hourly_idx]["visibility"]  # type: ignore
        evapotranspiration = hourly_df.iloc[hourly_idx]["evapotranspiration"]  # type: ignore
        reference_evapotranspiration = hourly_df.iloc[hourly_idx][
            "reference_evapotranspiration"
        ]  # type: ignore
        vapour_pressure_deficit = hourly_df.iloc[hourly_idx]["vapour_pressure_deficit"]  # type: ignore
        wind_speed_10m = hourly_df.iloc[hourly_idx]["wind_speed_10m"]  # type: ignore
        wind_speed_80m = hourly_df.iloc[hourly_idx]["wind_speed_80m"]  # type: ignore
        wind_speed_120m = hourly_df.iloc[hourly_idx]["wind_speed_120m"]  # type: ignore
        wind_speed_180m = hourly_df.iloc[hourly_idx]["wind_speed_180m"]  # type: ignore
        wind_direction_10m = hourly_df.iloc[hourly_idx]["wind_direction_10m"]  # type: ignore
        wind_direction_80m = hourly_df.iloc[hourly_idx]["wind_direction_80m"]  # type: ignore
        wind_direction_120m = hourly_df.iloc[hourly_idx]["wind_direction_120m"]  # type: ignore
        wind_direction_180m = hourly_df.iloc[hourly_idx]["wind_direction_180m"]  # type: ignore
        wind_gusts = hourly_df.iloc[hourly_idx]["wind_gusts"]  # type: ignore
        temperature_80m = hourly_df.iloc[hourly_idx]["temperature_80m"]  # type: ignore
        temperature_120m = hourly_df.iloc[hourly_idx]["temperature_120m"]  # type: ignore
        temperature_180m = hourly_df.iloc[hourly_idx]["temperature_180m"]  # type: ignore
        soil_temperature = hourly_df.iloc[hourly_idx]["soil_temperature"]  # type: ignore
        soil_temperature_6cm = hourly_df.iloc[hourly_idx]["soil_temperature_6cm"]  # type: ignore
        soil_temperature_18cm = hourly_df.iloc[hourly_idx]["soil_temperature_18cm"]  # type: ignore
        soil_temperature_54cm = hourly_df.iloc[hourly_idx]["soil_temperature_54cm"]  # type: ignore
        soil_moisture = hourly_df.iloc[hourly_idx]["soil_moisture"]  # type: ignore
        soil_moisture_1cm = hourly_df.iloc[hourly_idx]["soil_moisture_1cm"]  # type: ignore
        soil_moisture_3cm = hourly_df.iloc[hourly_idx]["soil_moisture_3cm"]  # type: ignore
        soil_moisture_9cm = hourly_df.iloc[hourly_idx]["soil_moisture_9cm"]  # type: ignore
        soil_moisture_27cm = hourly_df.iloc[hourly_idx]["soil_moisture_27cm"]  # type: ignore

        daily_idx = daily_df.index.get_indexer([dt], method="nearest")[0]
        daily_weather_code = daily_df.iloc[daily_idx]["weather_code"]  # type: ignore
        temperature_2m_max = daily_df.iloc[daily_idx]["temperature_2m_max"]  # type: ignore
        temperature_2m_min = daily_df.iloc[daily_idx]["temperature_2m_min"]  # type: ignore
        apparent_temperature_max = daily_df.iloc[daily_idx]["apparent_temperature_max"]  # type: ignore
        apparent_temperature_min = daily_df.iloc[daily_idx]["apparent_temperature_min"]  # type: ignore
        sunrise = daily_df.iloc[daily_idx]["sunrise"]  # type: ignore
        sunset = daily_df.iloc[daily_idx]["sunset"]  # type: ignore
        daylight_duration = daily_df.iloc[daily_idx]["daylight_duration"]  # type: ignore
        sunshine_duration = daily_df.iloc[daily_idx]["sunshine_duration"]  # type: ignore
        uv_index_max = daily_df.iloc[daily_idx]["uv_index_max"]  # type: ignore
        uv_index_clear_sky_max = daily_df.iloc[daily_idx]["uv_index_clear_sky_max"]  # type: ignore
        rain_sum = daily_df.iloc[daily_idx]["rain_sum"]  # type: ignore
        showers_sum = daily_df.iloc[daily_idx]["showers_sum"]  # type: ignore
        snowfall_sum = daily_df.iloc[daily_idx]["snowfall_sum"]  # type: ignore
        precipitation_sum = daily_df.iloc[daily_idx]["precipitation_sum"]  # type: ignore
        precipitation_hours = daily_df.iloc[daily_idx]["precipitation_hours"]  # type: ignore
        precipitation_probability_max = daily_df.iloc[daily_idx][
            "precipitation_probability_max"
        ]  # type: ignore
        wind_speed_10m_max = daily_df.iloc[daily_idx]["wind_speed_10m_max"]  # type: ignore
        wind_gusts_10m_max = daily_df.iloc[daily_idx]["wind_gusts_10m_max"]  # type: ignore
        wind_direction_10m_dominant = daily_df.iloc[daily_idx][
            "wind_direction_10m_dominant"
        ]  # type: ignore
        shortwave_radiation_sum = daily_df.iloc[daily_idx]["shortwave_radiation_sum"]  # type: ignore
        et0_fao_evapotranspiration = daily_df.iloc[daily_idx][
            "et0_fao_evapotranspiration"
        ]  # type: ignore

        return WeatherModel(
            temperature=temperature,
            relative_humidity=relative_humidity,
            dew_point=dew_point_2m,
            apparent_temperature=apparent_temperature,
            precipitation_probability=precipitation_probability,
            precipitation=precipitation,
            rain=rain,
            showers=showers,
            snowfall=snowfall,
            snow_depth=snow_depth,
            weather_code=weather_code,
            sealevel_pressure=pressure_msl,
            surface_pressure=surface_pressure,
            cloud_cover_total=cloud_cover,
            cloud_cover_low=cloud_cover_low,
            cloud_cover_mid=cloud_cover_mid,
            cloud_cover_high=cloud_cover_high,
            visibility=visibility,
            evapotranspiration=evapotranspiration,
            reference_evapotranspiration=reference_evapotranspiration,
            vapour_pressure_deficit=vapour_pressure_deficit,
            wind_speed_10m=wind_speed_10m,
            wind_speed_80m=wind_speed_80m,
            wind_speed_120m=wind_speed_120m,
            wind_speed_180m=wind_speed_180m,
            wind_direction_10m=wind_direction_10m,
            wind_direction_80m=wind_direction_80m,
            wind_direction_120m=wind_direction_120m,
            wind_direction_180m=wind_direction_180m,
            wind_gusts=wind_gusts,
            temperature_80m=temperature_80m,
            temperature_120m=temperature_120m,
            temperature_180m=temperature_180m,
            soil_temperature_0cm=soil_temperature,
            soil_temperature_6cm=soil_temperature_6cm,
            soil_temperature_18cm=soil_temperature_18cm,
            soil_temperature_54cm=soil_temperature_54cm,
            soil_moisture_0cm=soil_moisture,
            soil_moisture_1cm=soil_moisture_1cm,
            soil_moisture_3cm=soil_moisture_3cm,
            soil_moisture_9cm=soil_moisture_9cm,
            soil_moisture_27cm=soil_moisture_27cm,
            daily_weather_code=daily_weather_code,
            daily_maximum_temperature_2m=temperature_2m_max,
            daily_minimum_temperature_2m=temperature_2m_min,
            daily_maximum_apparent_temperature_2m=apparent_temperature_max,
            daily_minimum_apparent_temperature_2m=apparent_temperature_min,
            sunrise=sunrise,
            sunset=sunset,
            daylight_duration=daylight_duration,
            sunshine_duration=sunshine_duration,
            uv_index=uv_index_max,
            uv_index_clear_sky=uv_index_clear_sky_max,
            rain_sum=rain_sum,
            showers_sum=showers_sum,
            snowfall_sum=snowfall_sum,
            precipitation_sum=precipitation_sum,
            precipitation_hours=precipitation_hours,
            precipitation_probability_max=precipitation_probability_max,
            maximum_wind_speed_10m=wind_speed_10m_max,
            maximum_wind_gusts_10m=wind_gusts_10m_max,
            dominant_wind_direction=wind_direction_10m_dominant,
            shortwave_radiation_sum=shortwave_radiation_sum,
            daily_reference_evapotranspiration=et0_fao_evapotranspiration,
            version=version,
        )
    except (
        struct.error,
        requests.exceptions.HTTPError,
        requests.exceptions.ConnectionError,
    ):
        return None


def _create_openmeteo_weather_model(
    session: requests_cache.CachedSession,
    latitude: float,
    longitude: float,
    dt: datetime.datetime,
    tz: str,
    version: str,
) -> WeatherModel | None:
    # pylint: disable=broad-exception-caught
    client = openmeteo_requests.Client(session=session)
    try:
        params = {
            "latitude": latitude,
            "longitude": longitude,
            "hourly": [
                "temperature_2m",
                "relative_humidity_2m",
                "dew_point_2m",
                "apparent_temperature",
                "precipitation",
                "rain",
                "snowfall",
                "snow_depth",
                "weather_code",
                "pressure_msl",
                "surface_pressure",
                "cloud_cover",
                "cloud_cover_low",
                "cloud_cover_mid",
                "cloud_cover_high",
                "et0_fao_evapotranspiration",
                "vapour_pressure_deficit",
                "wind_speed_10m",
                "wind_speed_100m",
                "wind_direction_10m",
                "wind_direction_100m",
                "wind_gusts_10m",
                "soil_temperature_0_to_7cm",
                "soil_temperature_7_to_28cm",
                "soil_temperature_28_to_100cm",
                "soil_temperature_100_to_255cm",
                "soil_moisture_0_to_7cm",
                "soil_moisture_7_to_28cm",
                "soil_moisture_28_to_100cm",
                "soil_moisture_100_to_255cm",
            ],
            "daily": [
                "weather_code",
                "temperature_2m_max",
                "temperature_2m_min",
                "temperature_2m_mean",
                "apparent_temperature_max",
                "apparent_temperature_min",
                "apparent_temperature_mean",
                "sunrise",
                "sunset",
                "daylight_duration",
                "sunshine_duration",
                "precipitation_sum",
                "rain_sum",
                "snowfall_sum",
                "precipitation_hours",
                "wind_speed_10m_max",
                "wind_gusts_10m_max",
                "wind_direction_10m_dominant",
                "shortwave_radiation_sum",
                "et0_fao_evapotranspiration",
            ],
            "timezone": tz,
        }
        url = "https://historical-forecast-api.open-meteo.com/v1/forecast"
        if dt.date() > datetime.datetime.today().date():
            url = "https://api.open-meteo.com/v1/forecast"
            params["forecast_days"] = (
                datetime.datetime.today().date() - dt.date()
            ).days
        else:
            params["start_date"] = str((dt - datetime.timedelta(days=1.0)).date())
            params["end_date"] = str(dt.date())
        responses = client.weather_api(
            url,
            params=params,
        )
        return _parse_openmeteo(responses, tz, dt, version)
    except (
        requests.exceptions.RetryError,
        OpenMeteoRequestsError,
        requests.exceptions.ReadTimeout,
        requests.exceptions.HTTPError,
        requests.exceptions.ConnectionError,
    ):
        return None
    except Exception as e:
        e_text = str(e)
        if "Parameter 'start_date' is out of allowed range from" in e_text:
            return None
        raise e


@MEMORY.cache(ignore=["session"])
def _cached_create_openmeteo_weather_model(
    session: requests_cache.CachedSession,
    latitude: float,
    longitude: float,
    dt: datetime.datetime,
    tz: str,
    version: str,
) -> WeatherModel | None:
    return _create_openmeteo_weather_model(
        session, latitude, longitude, dt, tz, version
    )


def create_openmeteo_weather_model(
    session: requests_cache.CachedSession,
    latitude: float,
    longitude: float,
    dt: datetime.datetime,
    tz: str,
) -> WeatherModel | None:
    """Create a weather model from openmeteo."""
    if not pytest_is_running.is_running() and dt < datetime.datetime.now().replace(
        tzinfo=dt.tzinfo
    ) - datetime.timedelta(days=3):
        return _cached_create_openmeteo_weather_model(
            session, latitude, longitude, dt, tz, VERSION
        )
    with session.cache_disabled():
        return _create_openmeteo_weather_model(
            session, latitude, longitude, dt, tz, VERSION
        )



"""TennisData league model."""

import urllib.parse
from io import BytesIO
from typing import Any, Iterator
from urllib.parse import urlparse

import tqdm
from bs4 import BeautifulSoup
from openpyxl import load_workbook
from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ..game_model import GameModel
from ..league import League
from ..league_model import SHUTDOWN_FLAG, LeagueModel, needs_shutdown
from .tennisdata_game_model import create_tennisdata_game_model


class TennisDataLeagueModel(LeagueModel):
    """TennisData implementation of the league model."""

    def __init__(
        self,
        league: League,
        session: ScrapeSession,
        position: int | None = None,
    ) -> None:
        super().__init__(league, session, position=position)

    @classmethod
    def name(cls) -> str:
        """The name of the league model."""
        return "tennisdata-league-model"

    @classmethod
    def position_validator(cls) -> dict[str, str]:
        """Tennis position validators."""
        return {}

    def _row_to_game(self, row: Any) -> GameModel | None:
        tour_number_cell = str(row[0].value)
        if tour_number_cell in {"ATP", "None", "WTA"}:
            return None
        location_cell = str(row[1].value).strip()
        date_cell = str(row[3].value).strip()
        court_cell = str(row[5].value).strip()
        surface_cell = str(row[6].value).strip()
        best_of_cell = str(row[8].value).strip()
        winner_cell = str(row[9].value).strip()
        loser_cell = str(row[10].value).strip()
        winner_rank_cell = (
            str(row[11].value).strip()
            if row[11].value is not None and row[11].value != "N/A"
            else None
        )
        loser_rank_cell = (
            str(row[12].value).strip()
            if row[12].value is not None and row[12].value != "N/A"
            else None
        )
        winner_total_points_cell = (
            str(row[13].value).strip()
            if row[13].value is not None and row[13].value != "N/A"
            else None
        )
        loser_total_points_cell = (
            str(row[14].value).strip()
            if row[14].value is not None and row[14].value != "N/A"
            else None
        )
        winner_points_set_1_cell = (
            str(row[15].value).strip() if row[15].value is not None else None
        )
        loser_points_set_1_cell = (
            str(row[16].value).strip() if row[16].value is not None else None
        )
        winner_points_set_2_cell = (
            str(row[17].value).strip() if row[17].value is not None else None
        )
        loser_points_set_2_cell = (
            str(row[18].value).strip() if row[18].value is not None else None
        )
        winner_points_set_3_cell = (
            str(row[19].value).strip() if row[19].value is not None else None
        )
        loser_points_set_3_cell = (
            str(row[20].value).strip() if row[20].value is not None else None
        )
        current_cell = 21
        winner_points_set_4_cell = None
        loser_points_set_4_cell = None
        winner_points_set_5_cell = None
        loser_points_set_5_cell = None
        if self.league == League.ATP:
            winner_points_set_4_cell = (
                str(row[current_cell].value).strip()
                if row[current_cell].value is not None
                else None
            )
            current_cell += 1
            loser_points_set_4_cell = (
                str(row[current_cell].value).strip()
                if row[current_cell].value is not None
                else None
            )
            current_cell += 1
            winner_points_set_5_cell = (
                str(row[current_cell].value).strip()
                if row[current_cell].value is not None
                else None
            )
            current_cell += 1
            loser_points_set_5_cell = (
                str(row[current_cell].value).strip()
                if row[current_cell].value is not None
                else None
            )
            current_cell += 1
        winner_sets_cell = (
            str(row[current_cell].value).strip()
            if row[current_cell].value is not None
            else None
        )
        current_cell += 1
        loser_sets_cell = (
            str(row[current_cell].value).strip()
            if row[current_cell].value is not None
            else None
        )
        current_cell += 2

        winner_odds_cell = None
        if row[current_cell].value is not None:
            winner_odds_cell = str(row[current_cell].value).strip()
            current_cell += 1
        else:
            current_cell += 6
            winner_odds_cell = str(row[current_cell].value).strip()
            current_cell += 1

        loser_odds_cell: str | None = str(row[current_cell].value).strip()
        if winner_odds_cell == "None":
            winner_odds_cell = None
        if loser_odds_cell == "None":
            loser_odds_cell = None

        current_cell += 1
        return create_tennisdata_game_model(
            location=location_cell,
            date=date_cell,
            court=court_cell,
            surface=surface_cell,
            best_of=best_of_cell,
            winner=winner_cell,
            loser=loser_cell,
            winner_rank=winner_rank_cell,
            loser_rank=loser_rank_cell,
            winner_total_points=winner_total_points_cell,
            loser_total_points=loser_total_points_cell,
            winner_points_set_1=winner_points_set_1_cell,
            loser_points_set_1=loser_points_set_1_cell,
            winner_points_set_2=winner_points_set_2_cell,
            loser_points_set_2=loser_points_set_2_cell,
            winner_points_set_3=winner_points_set_3_cell,
            loser_points_set_3=loser_points_set_3_cell,
            winner_points_set_4=winner_points_set_4_cell,
            loser_points_set_4=loser_points_set_4_cell,
            winner_points_set_5=winner_points_set_5_cell,
            loser_points_set_5=loser_points_set_5_cell,
            winner_sets=winner_sets_cell,
            loser_sets=loser_sets_cell,
            winner_odds=winner_odds_cell,
            loser_odds=loser_odds_cell,
            session=self.session,
            league=self.league,
        )

    @property
    def games(self) -> Iterator[GameModel]:
        """Find all the games."""
        with self.session.wayback_disabled():
            try:
                with tqdm.tqdm(position=self.position) as pbar:
                    url = "http://www.tennis-data.co.uk/alldata.php"
                    response = None
                    with self.session.cache_disabled():
                        self.session.cache.delete(urls=[url])
                        response = self.session.get(url)
                    response.raise_for_status()
                    soup = BeautifulSoup(response.text, "lxml")
                    spreadsheet_urls = []
                    for a in soup.find_all("a", href=True):
                        if needs_shutdown():
                            return
                        spreadsheet_url = urllib.parse.urljoin(url, a.get("href"))
                        if not spreadsheet_url.endswith(".xlsx"):
                            continue
                        o = urlparse(spreadsheet_url)
                        path = o.path.split("/")
                        if self.league == League.WTA and path[-2].endswith("w"):
                            spreadsheet_urls.append(spreadsheet_url)
                        elif self.league == League.ATP and not path[-2].endswith("w"):
                            spreadsheet_urls.append(spreadsheet_url)

                    for count, spreadsheet_url in enumerate(
                        sorted(spreadsheet_urls, reverse=True)
                    ):
                        response = None
                        if count == 0:
                            with self.session.cache_disabled():
                                self.session.cache.delete(urls=[spreadsheet_url])
                                response = self.session.get(spreadsheet_url)
                        else:
                            response = self.session.get(spreadsheet_url)
                        response.raise_for_status()
                        workbook = load_workbook(filename=BytesIO(response.content))
                        ws = workbook.active
                        if ws is None:
                            raise ValueError("ws is null.")
                        for row in ws.iter_rows():
                            game_model = self._row_to_game(row)
                            if game_model is not None:
                                pbar.update(1)
                                pbar.set_description(f"TennisData - {game_model.dt}")
                                yield game_model
            except Exception as exc:
                SHUTDOWN_FLAG.set()
                raise exc



"""ESPNCricInfo league model."""

# pylint: disable=line-too-long
import datetime
import urllib.parse
from typing import Iterator

import tqdm
from bs4 import BeautifulSoup
from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ...game_model import GameModel
from ...league import League
from ...league_model import SHUTDOWN_FLAG, LeagueModel, needs_shutdown
from ..position import Position
from .ipl_espncricinfo_game_model import create_espncricinfo_game_model


class ESPNCricInfoLeagueModel(LeagueModel):
    """ESPNCricInfo implementation of the league model."""

    def __init__(
        self,
        session: ScrapeSession,
        position: int | None = None,
    ) -> None:
        super().__init__(League.IPL, session, position=position)

    @classmethod
    def name(cls) -> str:
        """The name of the league model."""
        return "ipl-espncricinfo-league-model"

    @classmethod
    def position_validator(cls) -> dict[str, str]:
        """Cricket position validators."""
        return {str(x): str(x) for x in Position}

    def _produce_games(self, url: str, pbar: tqdm.tqdm) -> Iterator[GameModel]:
        response = self.session.get(url)
        response.raise_for_status()
        soup = BeautifulSoup(response.text, "lxml")
        for a in soup.find_all("a", href=True):
            game_url = urllib.parse.urljoin(url, a.get("href"))
            if game_url.endswith("/full-scorecard"):
                pbar.update(1)
                game_model = create_espncricinfo_game_model(
                    self.session, game_url, self.league, self.position_validator()
                )
                if game_model is None:
                    continue
                pbar.set_description(f"ESPNCricInfo {game_model.dt}")
                yield game_model

    @property
    def games(self) -> Iterator[GameModel]:
        """Find all the games."""
        dt = datetime.datetime.now().date() - datetime.timedelta(days=2)
        try:
            with tqdm.tqdm(position=self.position) as pbar:
                while dt.year > 1900:
                    if needs_shutdown():
                        return
                    url = f"https://www.espncricinfo.com/live-cricket-match-results?date={dt.strftime('%d-%m-%Y')}"
                    response = self.session.get(url)
                    response.raise_for_status()
                    soup = BeautifulSoup(response.text, "lxml")
                    for a in soup.find_all("a", href=True):
                        match_url = urllib.parse.urljoin(url, a.get("href"))
                        if match_url.endswith("/match-schedule-fixtures-and-results"):
                            yield from self._produce_games(match_url, pbar)
                    dt = dt - datetime.timedelta(days=1)
        except Exception as exc:
            SHUTDOWN_FLAG.set()
            raise exc



"""ATP combined league model."""

# pylint: disable=line-too-long
from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ...combined.combined_league_model import CombinedLeagueModel
from ...league import League
from ..espn.atp_espn_league_model import ATPESPNLeagueModel
from ..tennisabstract.atp_tennisabstract_league_model import \
    ATPTennisAbstractLeagueModel
from ..tennisdata.atp_tennisdata_league_model import ATPTennisDataLeagueModel

ATP_TEAM_IDENTITY_MAP: dict[str, str] = {}
ATP_VENUE_IDENTITY_MAP: dict[str, str] = {}
ATP_PLAYER_IDENTITY_MAP: dict[str, str] = {}


class ATPCombinedLeagueModel(CombinedLeagueModel):
    """ATP combined implementation of the league model."""

    def __init__(self, session: ScrapeSession, league_filter: str | None) -> None:
        super().__init__(
            session,
            League.ATP,
            [
                ATPTennisAbstractLeagueModel(session, position=0),
                ATPESPNLeagueModel(session, position=1),
                ATPTennisDataLeagueModel(session, position=2),
            ],
            league_filter,
        )

    @classmethod
    def team_identity_map(cls) -> dict[str, str]:
        return ATP_TEAM_IDENTITY_MAP

    @classmethod
    def venue_identity_map(cls) -> dict[str, str]:
        return ATP_VENUE_IDENTITY_MAP

    @classmethod
    def player_identity_map(cls) -> dict[str, str]:
        return ATP_PLAYER_IDENTITY_MAP

    @classmethod
    def name(cls) -> str:
        return "atp-combined-league-model"



"""Bundesliga FootballData league model."""

from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ...footballdata.footballdata_league_model import FootballDataLeagueModel
from ...league import League


class BundesligaFootballDataLeagueModel(FootballDataLeagueModel):
    """Bundesliga FootballData implementation of the league model."""

    def __init__(
        self,
        session: ScrapeSession,
        position: int | None = None,
    ) -> None:
        super().__init__(League.BUNDESLIGA, session, position=position)

    @classmethod
    def name(cls) -> str:
        """The name of the league model."""
        return "bundesliga-footballdata-league-model"



"""Tests for the ESPN player model class."""
import datetime
import os
import unittest

import requests_mock
from scrapesession.scrapesession import ScrapeSession
from sportsball.data.espn.espn_player_model import create_espn_player_model


class TestESPNPlayerModel(unittest.TestCase):

    def setUp(self):
        self._session = ScrapeSession(backend="memory")
        self.dir = os.path.dirname(__file__)

    def test_identifier(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        identifier = "a"
        statistics_url = "http://sports.core.api.espn.com/v2/sports/football/leagues/nfl/events/401671855/competitions/401671855/competitors/5/roster/16837/statistics/0?lang=en&region=us"
        athletes_url = "http://sports.core.api.espn.com/v2/sports/football/leagues/nfl/seasons/2024/athletes/16837?lang=en&region=us"
        position_url = "http://sports.core.api.espn.com/v2/sports/football/leagues/nfl/positions/32?lang=en&region=us"
        college_url = "http://sports.core.api.espn.com/v2/colleges/2287?lang=en&region=us"
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "16837_statistics.json"), "rb") as f:
                m.get(statistics_url, content=f.read())
            with open(os.path.join(self.dir, "16837_athletes.json"), "rb") as f:
                m.get(athletes_url, content=f.read())
            with open(os.path.join(self.dir, "32_positions.json"), "rb") as f:
                m.get(position_url, content=f.read())
            with open(os.path.join(self.dir, "2287_colleges.json"), "rb") as f:
                m.get(college_url, content=f.read())
            m.get('https://historical-forecast-api.open-meteo.com/v1/forecast?latitude=40.511&longitude=-88.993&hourly=temperature_2m&hourly=relative_humidity_2m&hourly=dew_point_2m&hourly=apparent_temperature&hourly=precipitation&hourly=rain&hourly=snowfall&hourly=snow_depth&hourly=weather_code&hourly=pressure_msl&hourly=surface_pressure&hourly=cloud_cover&hourly=cloud_cover_low&hourly=cloud_cover_mid&hourly=cloud_cover_high&hourly=et0_fao_evapotranspiration&hourly=vapour_pressure_deficit&hourly=wind_speed_10m&hourly=wind_speed_100m&hourly=wind_direction_10m&hourly=wind_direction_100m&hourly=wind_gusts_10m&hourly=soil_temperature_0_to_7cm&hourly=soil_temperature_7_to_28cm&hourly=soil_temperature_28_to_100cm&hourly=soil_temperature_100_to_255cm&hourly=soil_moisture_0_to_7cm&hourly=soil_moisture_7_to_28cm&hourly=soil_moisture_28_to_100cm&hourly=soil_moisture_100_to_255cm&daily=weather_code&daily=temperature_2m_max&daily=temperature_2m_min&daily=temperature_2m_mean&daily=apparent_temperature_max&daily=apparent_temperature_min&daily=apparent_temperature_mean&daily=sunrise&daily=sunset&daily=daylight_duration&daily=sunshine_duration&daily=precipitation_sum&daily=rain_sum&daily=snowfall_sum&daily=precipitation_hours&daily=wind_speed_10m_max&daily=wind_gusts_10m_max&daily=wind_direction_10m_dominant&daily=shortwave_radiation_sum&daily=et0_fao_evapotranspiration&timezone=America%2FChicago&start_date=2023-09-14&end_date=2023-09-15&format=flatbuffers')
            player_model = create_espn_player_model(
                session=self._session,
                player={
                    "playerId": identifier,
                    "period": 0,
                    "active": False,
                    "starter": True,
                    "forPlayerId": 0,
                    "jersey": "93",
                    "valid": True,
                    "athlete": {
                        "$ref": athletes_url,
                    },
                    "position": {
                        "$ref": position_url,
                    },
                    "statistics": {
                        "$ref": statistics_url,
                    },
                    "didNotPlay": False,
                    "displayName": "S. Harris",
                },
                dt=dt,
                positions_validator={"DT": "DT"},
            )

            self.assertEqual(player_model.identifier, identifier)

    def test_no_birth_date(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        identifier = "a"
        statistics_url = "http://sports.core.api.espn.com/v2/sports/football/leagues/nfl/events/401671855/competitions/401671855/competitors/5/roster/16837/statistics/0?lang=en&region=us"
        athletes_url = "http://sports.core.api.espn.com/v2/sports/football/leagues/college-football/seasons/2024/athletes/102597?lang=en&region=us"
        position_url = "http://sports.core.api.espn.com/v2/sports/football/leagues/nfl/positions/32?lang=en&region=us"
        college_url = "http://sports.core.api.espn.com/v2/colleges/264?lang=en&region=us"
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "16837_statistics.json"), "rb") as f:
                m.get(statistics_url, content=f.read())
            with open(os.path.join(self.dir, "102597_athletes.json"), "rb") as f:
                m.get(athletes_url, content=f.read())
            with open(os.path.join(self.dir, "32_positions.json"), "rb") as f:
                m.get(position_url, content=f.read())
            with open(os.path.join(self.dir, "264_colleges.json"), "rb") as f:
                m.get(college_url, content=f.read())
            m.get("https://historical-forecast-api.open-meteo.com/v1/forecast?latitude=39.218056&longitude=-76.069444&hourly=temperature_2m&hourly=relative_humidity_2m&hourly=dew_point_2m&hourly=apparent_temperature&hourly=precipitation&hourly=rain&hourly=snowfall&hourly=snow_depth&hourly=weather_code&hourly=pressure_msl&hourly=surface_pressure&hourly=cloud_cover&hourly=cloud_cover_low&hourly=cloud_cover_mid&hourly=cloud_cover_high&hourly=et0_fao_evapotranspiration&hourly=vapour_pressure_deficit&hourly=wind_speed_10m&hourly=wind_speed_100m&hourly=wind_direction_10m&hourly=wind_direction_100m&hourly=wind_gusts_10m&hourly=soil_temperature_0_to_7cm&hourly=soil_temperature_7_to_28cm&hourly=soil_temperature_28_to_100cm&hourly=soil_temperature_100_to_255cm&hourly=soil_moisture_0_to_7cm&hourly=soil_moisture_7_to_28cm&hourly=soil_moisture_28_to_100cm&hourly=soil_moisture_100_to_255cm&daily=weather_code&daily=temperature_2m_max&daily=temperature_2m_min&daily=temperature_2m_mean&daily=apparent_temperature_max&daily=apparent_temperature_min&daily=apparent_temperature_mean&daily=sunrise&daily=sunset&daily=daylight_duration&daily=sunshine_duration&daily=precipitation_sum&daily=rain_sum&daily=snowfall_sum&daily=precipitation_hours&daily=wind_speed_10m_max&daily=wind_gusts_10m_max&daily=wind_direction_10m_dominant&daily=shortwave_radiation_sum&daily=et0_fao_evapotranspiration&timezone=America%2FNew_York&start_date=2023-09-14&end_date=2023-09-15&format=flatbuffers")
            m.get("https://historical-forecast-api.open-meteo.com/v1/forecast?latitude=47.0&longitude=-120.0&hourly=temperature_2m&hourly=relative_humidity_2m&hourly=dew_point_2m&hourly=apparent_temperature&hourly=precipitation&hourly=rain&hourly=snowfall&hourly=snow_depth&hourly=weather_code&hourly=pressure_msl&hourly=surface_pressure&hourly=cloud_cover&hourly=cloud_cover_low&hourly=cloud_cover_mid&hourly=cloud_cover_high&hourly=et0_fao_evapotranspiration&hourly=vapour_pressure_deficit&hourly=wind_speed_10m&hourly=wind_speed_100m&hourly=wind_direction_10m&hourly=wind_direction_100m&hourly=wind_gusts_10m&hourly=soil_temperature_0_to_7cm&hourly=soil_temperature_7_to_28cm&hourly=soil_temperature_28_to_100cm&hourly=soil_temperature_100_to_255cm&hourly=soil_moisture_0_to_7cm&hourly=soil_moisture_7_to_28cm&hourly=soil_moisture_28_to_100cm&hourly=soil_moisture_100_to_255cm&daily=weather_code&daily=temperature_2m_max&daily=temperature_2m_min&daily=temperature_2m_mean&daily=apparent_temperature_max&daily=apparent_temperature_min&daily=apparent_temperature_mean&daily=sunrise&daily=sunset&daily=daylight_duration&daily=sunshine_duration&daily=precipitation_sum&daily=rain_sum&daily=snowfall_sum&daily=precipitation_hours&daily=wind_speed_10m_max&daily=wind_gusts_10m_max&daily=wind_direction_10m_dominant&daily=shortwave_radiation_sum&daily=et0_fao_evapotranspiration&timezone=America%2FLos_Angeles&start_date=2023-09-14&end_date=2023-09-15&format=flatbuffers")
            player_model = create_espn_player_model(
                session=self._session,
                player={
                    "playerId": identifier,
                    "period": 0,
                    "active": False,
                    "starter": True,
                    "forPlayerId": 0,
                    "jersey": "93",
                    "valid": True,
                    "athlete": {
                        "$ref": athletes_url,
                    },
                    "position": {
                        "$ref": position_url,
                    },
                    "statistics": {
                        "$ref": statistics_url,
                    },
                    "didNotPlay": False,
                    "displayName": "S. Harris",
                },
                dt=dt,
                positions_validator={"DT": "DT"},
            )

            self.assertIsNone(player_model.birth_date)

    def test_kick_extra_points(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        identifier = "4565184"
        statistics_url = "http://sports.core.api.espn.com/v2/sports/football/leagues/college-football/events/401756846/competitions/401756846/competitors/2306/roster/4565184/statistics/0?lang=en&region=us"
        athletes_url = "http://sports.core.api.espn.com/v2/sports/football/leagues/college-football/seasons/2025/athletes/4565184?lang=en&region=us"
        position_url = "http://sports.core.api.espn.com/v2/sports/football/leagues/college-football/positions/1?lang=en&region=us"
        college_url = "http://sports.core.api.espn.com/v2/colleges/2306?lang=en&region=us"
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "4565184_statistics.json"), "rb") as f:
                m.get(statistics_url, content=f.read())
            with open(os.path.join(self.dir, "4565184_athletes.json"), "rb") as f:
                m.get(athletes_url, content=f.read())
            with open(os.path.join(self.dir, "1_positions.json"), "rb") as f:
                m.get(position_url, content=f.read())
            with open(os.path.join(self.dir, "2306_colleges.json"), "rb") as f:
                m.get(college_url, content=f.read())
            m.get("https://historical-forecast-api.open-meteo.com/v1/forecast?latitude=39.1914&longitude=-96.5809&hourly=temperature_2m&hourly=relative_humidity_2m&hourly=dew_point_2m&hourly=apparent_temperature&hourly=precipitation&hourly=rain&hourly=snowfall&hourly=snow_depth&hourly=weather_code&hourly=pressure_msl&hourly=surface_pressure&hourly=cloud_cover&hourly=cloud_cover_low&hourly=cloud_cover_mid&hourly=cloud_cover_high&hourly=et0_fao_evapotranspiration&hourly=vapour_pressure_deficit&hourly=wind_speed_10m&hourly=wind_speed_100m&hourly=wind_direction_10m&hourly=wind_direction_100m&hourly=wind_gusts_10m&hourly=soil_temperature_0_to_7cm&hourly=soil_temperature_7_to_28cm&hourly=soil_temperature_28_to_100cm&hourly=soil_temperature_100_to_255cm&hourly=soil_moisture_0_to_7cm&hourly=soil_moisture_7_to_28cm&hourly=soil_moisture_28_to_100cm&hourly=soil_moisture_100_to_255cm&daily=weather_code&daily=temperature_2m_max&daily=temperature_2m_min&daily=temperature_2m_mean&daily=apparent_temperature_max&daily=apparent_temperature_min&daily=apparent_temperature_mean&daily=sunrise&daily=sunset&daily=daylight_duration&daily=sunshine_duration&daily=precipitation_sum&daily=rain_sum&daily=snowfall_sum&daily=precipitation_hours&daily=wind_speed_10m_max&daily=wind_gusts_10m_max&daily=wind_direction_10m_dominant&daily=shortwave_radiation_sum&daily=et0_fao_evapotranspiration&timezone=America%2FChicago&start_date=2023-09-14&end_date=2023-09-15&format=flatbuffers")
            player_model = create_espn_player_model(
                session=self._session,
                player={
                    "playerId": identifier,
                    "period": 0,
                    "active": False,
                    "starter": False,
                    "forPlayerId": 0,
                    "jersey": "5",
                    "valid": False,
                    "athlete": {
                        "$ref": athletes_url,
                    },
                    "position": {
                        "$ref": position_url,
                    },
                    "statistics": {
                        "$ref": statistics_url,
                    },
                    "didNotPlay": False,
                    "displayName": "J. Bradley",
                },
                dt=dt,
                positions_validator={"WR": "WR"},
            )

            self.assertEqual(player_model.kick_extra_points, 0.0)

    def test_attempts_in_box(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        identifier = "291609"
        statistics_url = "http://sports.core.api.espn.com/v2/sports/soccer/leagues/eng.1/events/740617/competitions/740617/competitors/331/roster/291609/statistics/0?lang=en&region=us"
        athletes_url = "http://sports.core.api.espn.com/v2/sports/soccer/leagues/eng.1/seasons/2025/athletes/291609?lang=en&region=us"
        position_url = "http://sports.core.api.espn.com/v2/sports/soccer/leagues/eng.1/positions/1?lang=en&region=us"
        college_url = "http://sports.core.api.espn.com/v2/colleges/2306?lang=en&region=us"
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "0_statistics.json"), "rb") as f:
                m.get(statistics_url, content=f.read())
            with open(os.path.join(self.dir, "291609_athletes.json"), "rb") as f:
                m.get(athletes_url, content=f.read())
            with open(os.path.join(self.dir, "1_positions-2.json"), "rb") as f:
                m.get(position_url, content=f.read())
            player_model = create_espn_player_model(
                session=self._session,
                player={
                    "playerId": identifier,
                    "period": 0,
                    "active": True,
                    "starter": True,
                    "jersey": "1",
                    "athlete": {
                        "$ref": athletes_url,
                    },
                    "position": {
                        "$ref": position_url,
                    },
                    "statistics": {
                        "$ref": statistics_url,
                    },
                    "subbedIn": {
                        "didSub": False,
                    },
                    "subbedOut": {
                        "didSub": False
                    },
                    "formationPlace": "1",
                },
                dt=dt,
                positions_validator={"G": "G"},
            )

            self.assertEqual(player_model.attempts_in_box, 8)

    def test_second_assists(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        identifier = "141438"
        statistics_url = "http://sports.core.api.espn.com/v2/sports/soccer/leagues/fifa.world/events/633826/competitions/633826/competitors/628/roster/141438/statistics/0?lang=en&region=us"
        athletes_url = "http://sports.core.api.espn.com/v2/sports/soccer/leagues/fifa.world/seasons/2022/athletes/141438?lang=en&region=us"
        position_url = "http://sports.core.api.espn.com/v2/sports/soccer/leagues/fifa.world/positions/1?lang=en&region=us"
        college_url = "http://sports.core.api.espn.com/v2/colleges/2306?lang=en&region=us"
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "0_statistics-2.json"), "rb") as f:
                m.get(statistics_url, content=f.read())
            with open(os.path.join(self.dir, "141438_athletes.json"), "rb") as f:
                m.get(athletes_url, content=f.read())
            with open(os.path.join(self.dir, "1_positions-3.json"), "rb") as f:
                m.get(position_url, content=f.read())
            player_model = create_espn_player_model(
                session=self._session,
                player={
                    "playerId": identifier,
                    "period": 0,
                    "active": True,
                    "starter": True,
                    "jersey": "1",
                    "athlete": {
                        "$ref": athletes_url,
                    },
                    "position": {
                        "$ref": position_url,
                    },
                    "statistics": {
                        "$ref": statistics_url,
                    },
                    "subbedIn": {
                        "didSub": False,
                    },
                    "subbedOut": {
                        "didSub": False
                    },
                    "formationPlace": "1",
                },
                dt=dt,
                positions_validator={"G": "G"},
            )
            self.assertEqual(player_model.second_assists, 0.0)

    def test_qbr(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        identifier = "4870857"
        statistics_url = "http://sports.core.api.espn.com/v2/sports/football/leagues/college-football/events/401756846/competitions/401756846/competitors/2306/roster/4870857/statistics/0?lang=en&region=us"
        athletes_url = "http://sports.core.api.espn.com/v2/sports/football/leagues/college-football/seasons/2025/athletes/4870857?lang=en&region=us"
        position_url = "http://sports.core.api.espn.com/v2/sports/football/leagues/college-football/positions/8?lang=en&region=us"
        college_url = "http://sports.core.api.espn.com/v2/colleges/2306?lang=en&region=us"
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "0_statistics-3.json"), "rb") as f:
                m.get(statistics_url, content=f.read())
            with open(os.path.join(self.dir, "4870857_athletes.json"), "rb") as f:
                m.get(athletes_url, content=f.read())
            with open(os.path.join(self.dir, "8_positions.json"), "rb") as f:
                m.get(position_url, content=f.read())
            with open(os.path.join(self.dir, "2306_colleges.json"), "rb") as f:
                m.get(college_url, content=f.read())
            player_model = create_espn_player_model(
                session=self._session,
                player={
                    "playerId": identifier,
                    "period": 0,
                    "active": False,
                    "starter": False,
                    "forPlayerId": 0,
                    "jersey": "2",
                    "valid": False,
                    "athlete": {
                        "$ref": athletes_url,
                    },
                    "position": {
                        "$ref": position_url,
                    },
                    "statistics": {
                        "$ref": statistics_url,
                    },
                    "didNotPlay": False,
                    "displayName": "Av. Johnson",
                },
                dt=dt,
                positions_validator={"QB": "QB"},
            )
            self.assertEqual(player_model.qbr, 72.85)

    def test_turnover_points(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        identifier = "4433569"
        statistics_url = "http://sports.core.api.espn.com/v2/sports/basketball/leagues/mens-college-basketball/events/401708406/competitions/401708406/competitors/2/roster/4433569/statistics/0?lang=en&region=us"
        athletes_url = "http://sports.core.api.espn.com/v2/sports/basketball/leagues/mens-college-basketball/seasons/2025/athletes/4433569?lang=en&region=us"
        position_url = "http://sports.core.api.espn.com/v2/sports/basketball/leagues/mens-college-basketball/positions/2?lang=en&region=us"
        college_url = "http://sports.core.api.espn.com/v2/colleges/2?lang=en&region=us"
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "0_statistics-4.json"), "rb") as f:
                m.get(statistics_url, content=f.read())
            with open(os.path.join(self.dir, "4433569_athletes.json"), "rb") as f:
                m.get(athletes_url, content=f.read())
            with open(os.path.join(self.dir, "2_positions.json"), "rb") as f:
                m.get(position_url, content=f.read())
            with open(os.path.join(self.dir, "2_colleges.json"), "rb") as f:
                m.get(college_url, content=f.read())
            m.get("https://historical-forecast-api.open-meteo.com/v1/forecast?latitude=32.603&longitude=-85.486&hourly=temperature_2m&hourly=relative_humidity_2m&hourly=dew_point_2m&hourly=apparent_temperature&hourly=precipitation&hourly=rain&hourly=snowfall&hourly=snow_depth&hourly=weather_code&hourly=pressure_msl&hourly=surface_pressure&hourly=cloud_cover&hourly=cloud_cover_low&hourly=cloud_cover_mid&hourly=cloud_cover_high&hourly=et0_fao_evapotranspiration&hourly=vapour_pressure_deficit&hourly=wind_speed_10m&hourly=wind_speed_100m&hourly=wind_direction_10m&hourly=wind_direction_100m&hourly=wind_gusts_10m&hourly=soil_temperature_0_to_7cm&hourly=soil_temperature_7_to_28cm&hourly=soil_temperature_28_to_100cm&hourly=soil_temperature_100_to_255cm&hourly=soil_moisture_0_to_7cm&hourly=soil_moisture_7_to_28cm&hourly=soil_moisture_28_to_100cm&hourly=soil_moisture_100_to_255cm&daily=weather_code&daily=temperature_2m_max&daily=temperature_2m_min&daily=temperature_2m_mean&daily=apparent_temperature_max&daily=apparent_temperature_min&daily=apparent_temperature_mean&daily=sunrise&daily=sunset&daily=daylight_duration&daily=sunshine_duration&daily=precipitation_sum&daily=rain_sum&daily=snowfall_sum&daily=precipitation_hours&daily=wind_speed_10m_max&daily=wind_gusts_10m_max&daily=wind_direction_10m_dominant&daily=shortwave_radiation_sum&daily=et0_fao_evapotranspiration&timezone=America%2FChicago&start_date=2023-09-14&end_date=2023-09-15&format=flatbuffers")
            player_model = create_espn_player_model(
                session=self._session,
                player={
                    "playerId": identifier,
                    "period": 0,
                    "active": False,
                    "starter": True,
                    "forPlayerId": 0,
                    "jersey": "4",
                    "valid": False,
                    "athlete": {
                        "$ref": athletes_url,
                    },
                    "position": {
                        "$ref": position_url,
                    },
                    "statistics": {
                        "$ref": statistics_url,
                    },
                    "didNotPlay": False,
                    "displayName": "J. Broome",
                    "ejected": False,
                },
                dt=dt,
                positions_validator={"F": "F"},
            )
            self.assertEqual(player_model.turnover_points, 0.0)

    def test_penalty_kill_percentage(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        identifier = "5767"
        statistics_url = "http://sports.core.api.espn.com/v2/sports/hockey/leagues/nhl/events/401685276/competitions/401685276/competitors/16/roster/5767/statistics/0?lang=en&region=us"
        athletes_url = "http://sports.core.api.espn.com/v2/sports/hockey/leagues/nhl/seasons/2025/athletes/5767?lang=en&region=us"
        position_url = "http://sports.core.api.espn.com/v2/sports/hockey/leagues/nhl/positions/3?lang=en&region=us"
        college_url = "http://sports.core.api.espn.com/v2/colleges/103?lang=en&region=us"
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "0_statistics-6.json"), "rb") as f:
                m.get(statistics_url, content=f.read())
            with open(os.path.join(self.dir, "5767_athletes.json"), "rb") as f:
                m.get(athletes_url, content=f.read())
            with open(os.path.join(self.dir, "3_positions.json"), "rb") as f:
                m.get(position_url, content=f.read())
            with open(os.path.join(self.dir, "103_colleges.json"), "rb") as f:
                m.get(college_url, content=f.read())
            m.get("https://historical-forecast-api.open-meteo.com/v1/forecast?latitude=42.335&longitude=-71.170278&hourly=temperature_2m&hourly=relative_humidity_2m&hourly=dew_point_2m&hourly=apparent_temperature&hourly=precipitation&hourly=rain&hourly=snowfall&hourly=snow_depth&hourly=weather_code&hourly=pressure_msl&hourly=surface_pressure&hourly=cloud_cover&hourly=cloud_cover_low&hourly=cloud_cover_mid&hourly=cloud_cover_high&hourly=et0_fao_evapotranspiration&hourly=vapour_pressure_deficit&hourly=wind_speed_10m&hourly=wind_speed_100m&hourly=wind_direction_10m&hourly=wind_direction_100m&hourly=wind_gusts_10m&hourly=soil_temperature_0_to_7cm&hourly=soil_temperature_7_to_28cm&hourly=soil_temperature_28_to_100cm&hourly=soil_temperature_100_to_255cm&hourly=soil_moisture_0_to_7cm&hourly=soil_moisture_7_to_28cm&hourly=soil_moisture_28_to_100cm&hourly=soil_moisture_100_to_255cm&daily=weather_code&daily=temperature_2m_max&daily=temperature_2m_min&daily=temperature_2m_mean&daily=apparent_temperature_max&daily=apparent_temperature_min&daily=apparent_temperature_mean&daily=sunrise&daily=sunset&daily=daylight_duration&daily=sunshine_duration&daily=precipitation_sum&daily=rain_sum&daily=snowfall_sum&daily=precipitation_hours&daily=wind_speed_10m_max&daily=wind_gusts_10m_max&daily=wind_direction_10m_dominant&daily=shortwave_radiation_sum&daily=et0_fao_evapotranspiration&timezone=America%2FNew_York&start_date=2023-09-14&end_date=2023-09-15&format=flatbuffers")
            player_model = create_espn_player_model(
                session=self._session,
                player={
                    "playerId": identifier,
                    "jersey": "13",
                    "athlete": {
                        "$ref": athletes_url,
                    },
                    "position": {
                        "$ref": position_url,
                    },
                    "statistics": {
                        "$ref": statistics_url,
                    },
                    "displayName": "K. Hayes",
                    "scratched": False,
                },
                dt=dt,
                positions_validator={"RW": "RW"},
            )
            self.assertEqual(player_model.penalty_kill_percentage, 0.0)

    def test_team_rebounds(self):
        dt = datetime.datetime(2023, 9, 15, 0, 15)
        identifier = "3142055"
        statistics_url = "http://sports.core.api.espn.com/v2/sports/basketball/leagues/wnba/events/401736120/competitions/401736120/competitors/3/roster/3142055/statistics/0?lang=en&region=us"
        athletes_url = "http://sports.core.api.espn.com/v2/sports/basketball/leagues/wnba/seasons/2025/athletes/3142055?lang=en&region=us"
        position_url = "http://sports.core.api.espn.com/v2/sports/basketball/leagues/wnba/positions/7?lang=en&region=us"
        college_url = "http://sports.core.api.espn.com/v2/colleges/97?lang=en&region=us"
        with requests_mock.Mocker() as m:
            with open(os.path.join(self.dir, "0_statistics-15.json"), "rb") as f:
                m.get(statistics_url, content=f.read())
            with open(os.path.join(self.dir, "3142055_athletes.json"), "rb") as f:
                m.get(athletes_url, content=f.read())
            with open(os.path.join(self.dir, "7_positions.json"), "rb") as f:
                m.get(position_url, content=f.read())
            with open(os.path.join(self.dir, "97_colleges.json"), "rb") as f:
                m.get(college_url, content=f.read())
            m.get("https://historical-forecast-api.open-meteo.com/v1/forecast?latitude=38.256111&longitude=-85.751389&hourly=temperature_2m&hourly=relative_humidity_2m&hourly=dew_point_2m&hourly=apparent_temperature&hourly=precipitation&hourly=rain&hourly=snowfall&hourly=snow_depth&hourly=weather_code&hourly=pressure_msl&hourly=surface_pressure&hourly=cloud_cover&hourly=cloud_cover_low&hourly=cloud_cover_mid&hourly=cloud_cover_high&hourly=et0_fao_evapotranspiration&hourly=vapour_pressure_deficit&hourly=wind_speed_10m&hourly=wind_speed_100m&hourly=wind_direction_10m&hourly=wind_direction_100m&hourly=wind_gusts_10m&hourly=soil_temperature_0_to_7cm&hourly=soil_temperature_7_to_28cm&hourly=soil_temperature_28_to_100cm&hourly=soil_temperature_100_to_255cm&hourly=soil_moisture_0_to_7cm&hourly=soil_moisture_7_to_28cm&hourly=soil_moisture_28_to_100cm&hourly=soil_moisture_100_to_255cm&daily=weather_code&daily=temperature_2m_max&daily=temperature_2m_min&daily=temperature_2m_mean&daily=apparent_temperature_max&daily=apparent_temperature_min&daily=apparent_temperature_mean&daily=sunrise&daily=sunset&daily=daylight_duration&daily=sunshine_duration&daily=precipitation_sum&daily=rain_sum&daily=snowfall_sum&daily=precipitation_hours&daily=wind_speed_10m_max&daily=wind_gusts_10m_max&daily=wind_direction_10m_dominant&daily=shortwave_radiation_sum&daily=et0_fao_evapotranspiration&timezone=America%2FKentucky%2FLouisville&start_date=2023-09-14&end_date=2023-09-15&format=flatbuffers")
            player_model = create_espn_player_model(
                session=self._session,
                player={
                    "playerId": identifier,
                    "jersey": "13",
                    "athlete": {
                        "$ref": athletes_url,
                    },
                    "position": {
                        "$ref": position_url,
                    },
                    "statistics": {
                        "$ref": statistics_url,
                    },
                    "displayName": "K. Hayes",
                    "scratched": False,
                },
                dt=dt,
                positions_validator={"F": "F"},
            )
            self.assertEqual(player_model.team_rebounds, 0.0)



"""WNBA WNBA.com game model."""

import re
from urllib.parse import urlparse

from bs4 import BeautifulSoup
from dateutil.parser import parse
from playwright.sync_api import Playwright
from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ....playwright import ensure_install
from ...game_model import GameModel
from ...league import League
from ...team_model import VERSION
from ...venue_model import VERSION as VENUE_VERSION
from .wnba_wnbacom_team_model import create_wnba_wnbacom_team_model
from .wnba_wnbacom_venue_model import create_wnba_wnbacom_venue_model


def create_wnba_wnbacom_game_model(
    url: str,
    session: ScrapeSession,
    playwright: Playwright,
    version: str,
) -> GameModel:
    """Create a game model from WNBA.com."""
    o = urlparse(url)
    end_path_split = o.path.split("/")[-1].split("-")
    week = int(end_path_split[-1])

    ensure_install()
    browser = playwright.chromium.launch()
    context = browser.new_context()
    page = context.new_page()
    page.goto(url, wait_until="load")
    soup = BeautifulSoup(page.content(), "lxml")

    dt = None
    for div_date in soup.find_all(
        "div", {"class": re.compile("_GameStatusExpanded__date.*")}
    ):
        date_txt = div_date.get_text().strip()
        for div_time in soup.find_all(
            "div", {"class": re.compile("_GameStatusExpanded__date.*")}
        ):
            time_txt = div_time.get_text().strip()
            dt = parse(" ".join([date_txt, time_txt]))
            break
        break
    if dt is None:
        raise ValueError("dt is null")

    venue_model = None
    for div_location in soup.find_all(
        "div", {"class": re.compile("_GameDetailsHeader--location.*")}
    ):
        venue_model = create_wnba_wnbacom_venue_model(
            venue_name=div_location.get_text().strip(),
            session=session,
            dt=dt,
            version=VENUE_VERSION,
        )
        break

    teams = []
    for div_team in soup.find_all(
        "div", {"class": re.compile("_GameDetailsHeader--team.*")}
    ):
        team_name = div_team.get_text().strip().replace("\n", "").strip()
        teams.append(
            create_wnba_wnbacom_team_model(
                team_name=team_name, dt=dt, session=session, version=VERSION
            )
        )

    return GameModel(
        dt=dt,
        week=week,
        game_number=None,
        venue=venue_model,
        teams=list(reversed(teams)),
        end_dt=None,
        attendance=None,
        league=League.WNBA,
        year=dt.year,
        season_type=None,
        postponed=None,
        play_off=None,
        distance=None,
        dividends=[],
        pot=None,
        version=version,
        umpires=[],
        best_of=None,
    )



"""Aussportsbetting venue model."""

# pylint: disable=duplicate-code
import datetime

from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ...cache import MEMORY
from ..google.google_address_model import create_google_address_model
from ..venue_model import VenueModel


@MEMORY.cache(ignore=["session"])
def create_aussportsbetting_venue_model(
    venue: str,
    session: ScrapeSession,
    dt: datetime.datetime,
    version: str,
) -> VenueModel:
    """Create a venue model based off aus sports betting."""
    address_model = create_google_address_model(venue, session, dt)
    return VenueModel(
        identifier=venue,
        name=venue,
        address=address_model,
        is_grass=None,
        is_indoor=None,
        is_turf=None,
        is_dirt=None,
        is_hard=None,
        version=version,
    )



"""Tests for the X social model class."""
import datetime
import os
import unittest

import requests_cache

from sportsball.data.x.x_social_model import create_x_social_model, X_API_KEY, X_API_SECRET_KEY, X_ACCESS_TOKEN, X_ACCESS_TOKEN_SECRET


class TestXSocialModel(unittest.TestCase):

    def setUp(self):
        self._session = requests_cache.CachedSession(backend="memory")
        self._social_models = create_x_social_model("", self._session, datetime.datetime(2010, 10, 10, 10, 10, 10))

    def test_empty(self):
        self.assertListEqual(self._social_models, [])

    def test_no_username(self):
        os.environ[X_API_KEY] = "api"
        os.environ[X_API_SECRET_KEY] = "api-secret"
        os.environ[X_ACCESS_TOKEN] = "access"
        os.environ[X_ACCESS_TOKEN_SECRET] = "access-secret"
        self._social_models = create_x_social_model(
            "Hawaii Rainbow Warriors Men's",
            self._session,
            datetime.datetime(2010, 10, 10, 10, 10, 10),
        )
        del os.environ[X_ACCESS_TOKEN_SECRET]
        del os.environ[X_ACCESS_TOKEN]
        del os.environ[X_API_SECRET_KEY]
        del os.environ[X_API_KEY]



"""Footballdata game model."""

# pylint: disable=too-many-arguments,duplicate-code
import datetime
import logging

import pytest_is_running
from dateutil import parser
from dateutil.parser import parse
from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ...cache import MEMORY
from ..game_model import VERSION as GAME_VERSION
from ..game_model import GameModel
from ..league import League
from ..team_model import VERSION as TEAM_VERSION
from ..umpire_model import VERSION as UMPIRE_VERSION
from .footballdata_team_model import create_footballdata_team_model
from .footballdata_umpire_model import create_footballdata_umpire_model


def _create_footballdata_game_model(
    session: ScrapeSession,
    league: League,
    dt: datetime.datetime,
    home_team: str,
    away_team: str,
    full_time_home_goals: str,
    full_time_away_goals: str,
    referee: str | None,
    home_shots: str | None,
    away_shots: str | None,
    home_shots_on_target: str | None,
    away_shots_on_target: str | None,
    home_fouls: str | None,
    away_fouls: str | None,
    home_yellow_cards: str | None,
    away_yellow_cards: str | None,
    home_red_cards: str | None,
    away_red_cards: str | None,
    home_odds: str | None,
    away_odds: str | None,
    version: str,
) -> GameModel:
    home_team_model = create_footballdata_team_model(
        session=session,
        league=league,
        dt=dt,
        name=home_team,
        full_time_goals=full_time_home_goals,
        shots=home_shots,
        shots_on_target=home_shots_on_target,
        fouls=home_fouls,
        yellow_cards=home_yellow_cards,
        red_cards=home_red_cards,
        odds=home_odds,
        version=TEAM_VERSION,
    )
    away_team_model = create_footballdata_team_model(
        session=session,
        league=league,
        dt=dt,
        name=away_team,
        full_time_goals=full_time_away_goals,
        shots=away_shots,
        shots_on_target=away_shots_on_target,
        fouls=away_fouls,
        yellow_cards=away_yellow_cards,
        red_cards=away_red_cards,
        odds=away_odds,
        version=TEAM_VERSION,
    )
    return GameModel(
        dt=dt,
        week=None,
        game_number=None,
        venue=None,
        teams=[home_team_model, away_team_model],
        end_dt=None,
        attendance=None,
        league=str(league),
        year=None,
        season_type=None,
        postponed=None,
        play_off=None,
        distance=None,
        dividends=[],
        pot=None,
        version=version,
        umpires=[
            create_footballdata_umpire_model(name=referee, version=UMPIRE_VERSION)  # type: ignore
        ]
        if referee is not None
        else [],
        best_of=None,
    )


@MEMORY.cache(ignore=["session"])
def _cached_create_footballdata_game_model(
    session: ScrapeSession,
    league: League,
    dt: datetime.datetime,
    home_team: str,
    away_team: str,
    full_time_home_goals: str,
    full_time_away_goals: str,
    referee: str | None,
    home_shots: str | None,
    away_shots: str | None,
    home_shots_on_target: str | None,
    away_shots_on_target: str | None,
    home_fouls: str | None,
    away_fouls: str | None,
    home_yellow_cards: str | None,
    away_yellow_cards: str | None,
    home_red_cards: str | None,
    away_red_cards: str | None,
    home_odds: str | None,
    away_odds: str | None,
    version: str,
) -> GameModel:
    return _create_footballdata_game_model(
        session=session,
        league=league,
        dt=dt,
        home_team=home_team,
        away_team=away_team,
        full_time_home_goals=full_time_home_goals,
        full_time_away_goals=full_time_away_goals,
        referee=referee,
        home_shots=home_shots,
        away_shots=away_shots,
        home_shots_on_target=home_shots_on_target,
        away_shots_on_target=away_shots_on_target,
        home_fouls=home_fouls,
        away_fouls=away_fouls,
        home_yellow_cards=home_yellow_cards,
        away_yellow_cards=away_yellow_cards,
        home_red_cards=home_red_cards,
        away_red_cards=away_red_cards,
        home_odds=home_odds,
        away_odds=away_odds,
        version=version,
    )


def create_footballdata_game_model(
    session: ScrapeSession,
    league: League,
    date: str,
    time: str,
    home_team: str,
    away_team: str,
    full_time_home_goals: str,
    full_time_away_goals: str,
    referee: str | None,
    home_shots: str | None,
    away_shots: str | None,
    home_shots_on_target: str | None,
    away_shots_on_target: str | None,
    home_fouls: str | None,
    away_fouls: str | None,
    home_yellow_cards: str | None,
    away_yellow_cards: str | None,
    home_red_cards: str | None,
    away_red_cards: str | None,
    home_odds: str | None,
    away_odds: str | None,
) -> GameModel:
    """Create a game model based off footballdata."""
    dt = None
    try:
        dt = parse(" ".join([date, time]))
    except parser._parser.ParserError as exc:  # type: ignore
        logging.error(str(exc))
        logging.error("%s %s", date, time)
        raise exc
    if not pytest_is_running.is_running() and dt < datetime.datetime.now().replace(
        tzinfo=dt.tzinfo
    ) - datetime.timedelta(days=7):
        return _cached_create_footballdata_game_model(
            session=session,
            league=league,
            dt=dt,
            home_team=home_team,
            away_team=away_team,
            full_time_home_goals=full_time_home_goals,
            full_time_away_goals=full_time_away_goals,
            referee=referee,
            home_shots=home_shots,
            away_shots=away_shots,
            home_shots_on_target=home_shots_on_target,
            away_shots_on_target=away_shots_on_target,
            home_fouls=home_fouls,
            away_fouls=away_fouls,
            home_yellow_cards=home_yellow_cards,
            away_yellow_cards=away_yellow_cards,
            home_red_cards=home_red_cards,
            away_red_cards=away_red_cards,
            home_odds=home_odds,
            away_odds=away_odds,
            version=GAME_VERSION,
        )
    with session.cache_disabled():
        return _create_footballdata_game_model(
            session=session,
            league=league,
            dt=dt,
            home_team=home_team,
            away_team=away_team,
            full_time_home_goals=full_time_home_goals,
            full_time_away_goals=full_time_away_goals,
            referee=referee,
            home_shots=home_shots,
            away_shots=away_shots,
            home_shots_on_target=home_shots_on_target,
            away_shots_on_target=away_shots_on_target,
            home_fouls=home_fouls,
            away_fouls=away_fouls,
            home_yellow_cards=home_yellow_cards,
            away_yellow_cards=away_yellow_cards,
            home_red_cards=home_red_cards,
            away_red_cards=away_red_cards,
            home_odds=home_odds,
            away_odds=away_odds,
            version=GAME_VERSION,
        )
